package common;

import java.nio.ByteBuffer;
import java.util.Random;


public class BitStream extends MuxBuffer {
	private static final int ALL_BITS_MASK=(int)0xffffffff;
	private static final int ESCAPE_MASK=0xfffffc;
	private static final byte ESCAPE_BYTE=0x03;
	private static int BIT_MASKS[]={128,64,32,16,8,4,2,1};
	
	/*private static final long ESCAPE_MASK0=0xfffffcl;
	private static final long ESCAPE_MASK1=0xfffffc00l;
	private static final long ESCAPE_MASK2=0xfffffc0000l;
	private static final long ESCAPE_MASK3=0xfffffc000000l;
	private static final long LOWER_BITS_MASK=0xffffffff;
	
	private static final int TOP_BYTES_MASK=(int)0xffffff00;
	private static final long ESCAPE_LONG=0x03;*/
	private int bitsUsed;
	//ByteBuffer buffer=ByteBuffer.wrap(new byte[]{0x55,0x55,0x55,0x55});
	//ByteBuffer buffer=ByteBuffer.wrap(new byte[]{0,0,0,0});
	//ByteBuffer buffer;
	public BitStream(byte buffer[]) {
		super(buffer);
	}
	public BitStream(int bytes) {
		super(bytes);
	}
	public BitStream(ByteBuffer buffer) {
		super(buffer);
	}
	
	public void testCopy() {
		
		byte source[]=new byte[50];
		byte target[]=new byte[source.length];
		Random random=new Random();
		random.nextBytes(source);
		
		/*for (int i=0;i<source.length;i++) {
			source[i]=(byte)(i);
		}*/
		buffer=ByteBuffer.wrap(source);
		BitStream out=new BitStream(target);
		int bits=0;
		for (int i=0;bits + i < source.length * 8 && i<26;i++) {
			out.setBits(i, getBits(i));
			bits+=i;
		}
		for (int i=0;i<bits/8;i++) {
			if (source[i]!=target[i]) {
				throw new RuntimeException("Failed!");
			}
		}
	}
	/*private int currentByte=0x55;
	
	private final void nextByte() {
		bitsUsed=0;
		currentByte=buffer.get() & 0xff;
	}*/
	/**
	 * Get a single bit.  Simplified for faster access
	 * @return
	 */
	public final int getBit() {
		int bit;
		if (bitsUsed==7) {
			bit=buffer.get() & 1;
			bitsUsed=0;
			//bit=currentByte & 1;
			//nextByte();
		} else {
			int mask=1<<(7-bitsUsed++);
			bit=buffer.get(buffer.position()) & mask;
			//bit=currentByte & mask;
			if (bit != 0) {
				bit=1;
			}
		}
		return bit;
	}
	/**
	 * Get up to 32 bits as an int.  25 bits is the max safe value.
	 * @param bits
	 * @return
	 */
	public int getBits(int bits) {
		int newBitsUsed=bitsUsed+bits;
		int mask=(1<<bits)-1;
		int value;
		if (newBitsUsed<=8) {
			int shift=8-newBitsUsed;
			mask <<=shift;
			//Optimization for byte level operations
			if (newBitsUsed==8) {
				value=buffer.get() & mask;
				bitsUsed=0;
			} else {
				value=buffer.get(buffer.position()) & mask;
				bitsUsed=newBitsUsed;
			}
			return value >>>=shift;
		} else if (newBitsUsed <=16) {
			int shift=16-newBitsUsed;
			mask <<=shift;
			value=buffer.getShort(buffer.position()) & mask;
			value >>>=shift;
		} else if (newBitsUsed <=32) {
			int shift=32-newBitsUsed;
			mask <<=shift;
			value=buffer.getInt(buffer.position()) & mask;
			value >>>=shift;
		} else {
			throw new RuntimeException("Exceeded bit reading capacity: " + bits);
		}
		buffer.position(buffer.position() + newBitsUsed/8);
		bitsUsed=newBitsUsed % 8;
		return value;
	}
	public void setBits(int bits, int value) {
		int mask=(1<<bits)-1;
		int newBitsUsed=bitsUsed+bits;
		
		if (newBitsUsed<=8) {
			int shift=8-newBitsUsed;
			mask <<=shift;
			int tmp = (buffer.get(buffer.position()) & (~mask)) | (value << shift);
			//Optimization for byte level operations
			if (newBitsUsed==8) {
				buffer.put((byte)tmp);
				bitsUsed=0;
			} else {
				buffer.put(buffer.position(),(byte)tmp);
				bitsUsed=newBitsUsed;
			}
			return;
		} else if (newBitsUsed <=16) {
			int shift=16-newBitsUsed;
			mask <<=shift;
			int tmp = (buffer.getShort(buffer.position()) & (~mask)) | (value << shift);
			buffer.putShort(buffer.position(), (short)tmp);
		} else if (newBitsUsed <=32) {
			int shift=32-newBitsUsed;
			mask <<=shift;
			int tmp = (buffer.getInt(buffer.position()) & (~mask)) | (value << shift);
			buffer.putInt(buffer.position(), tmp);
		} else {
			throw new RuntimeException("Exceeded bit writing capacity: " + bits);
		}

		buffer.position(buffer.position() + newBitsUsed/8);
		bitsUsed=newBitsUsed % 8;
	}
	/**
	 * Sets bits and zeros trailing bits
	 * @param bits
	 * @param value
	 */
	public void appendBits(int bits, int value) {
		int newBitsUsed=bitsUsed+bits;
		
		if (newBitsUsed<=8) {
			int shift=8-newBitsUsed;
			int mask=(1<<(bits + shift))-1;
			int tmp = (buffer.get(buffer.position()) & (~mask)) | (value << shift);
			//Optimization for byte level operations
			if (newBitsUsed==8) {
				buffer.put((byte)tmp);
				bitsUsed=0;
			} else {
				buffer.put(buffer.position(),(byte)tmp);
				bitsUsed=newBitsUsed;
			}
			return;
		} else if (newBitsUsed <=16) {
			int shift=16-newBitsUsed;
			int mask=(1<<(bits + shift))-1;
			int tmp = (buffer.getShort(buffer.position()) & (~mask)) | (value << shift);
			buffer.putShort(buffer.position(), (short)tmp);
		} else if (newBitsUsed <=32) {
			int shift=32-newBitsUsed;
			int mask=(1<<(bits + shift))-1;
			int tmp = (buffer.getInt(buffer.position()) & (~mask)) | (value << shift);
			buffer.putInt(buffer.position(), tmp);
		} else {
			throw new RuntimeException("Exceeded bit writing capacity: " + bits);
		}

		buffer.position(buffer.position() + newBitsUsed/8);
		bitsUsed=newBitsUsed % 8;
	}	
	public void skipBits(int bits) {
		int newBitsUsed=bitsUsed+bits;
		if (newBitsUsed < 0) {
			buffer.position(buffer.position() + newBitsUsed/8 - 1);
			bitsUsed=(newBitsUsed % 8) + 8;
		} else {
			buffer.position(buffer.position() + newBitsUsed/8);
			bitsUsed=newBitsUsed % 8;
		}
	}
	public byte getByte() {
		if (bitsUsed==0) {
			return buffer.get();
		}
		int value=(buffer.getShort(buffer.position()) & (0xff << bitsUsed));
		buffer.get();//Move one byte forward
		return (byte)(value >> bitsUsed);
	}
	public void setPosition(int position) {
		buffer.position(position);
		bitsUsed=0;
	}
	/*public void appendEscaped2(BitStream source, int end) {
		buffer.limit(buffer.capacity());
		if (source.bitsUsed > 0) {
			//byte align the source
			int bitsRemaining=8-source.bitsUsed;
			this.appendBits(bitsRemaining, source.getBits(bitsRemaining));
		}
		long sourceLong=ALL_BITS_MASK<<32;
		final ByteBuffer sourceBuffer=source.buffer;
		final int shift=8-bitsUsed;
		
		long myLong=(ALL_BITS_MASK << bitsUsed) | ((buffer.get(buffer.position())&0xff)>>>shift);
		
		final int myBitsUsed=bitsUsed;
		bitsUsed=0;//The shifting we are doing essentially zeros this out
		//Create local references for faster execution
		final ByteBuffer myBuffer=buffer;
		final int sourceLimit=sourceBuffer.limit();
		final int shiftedEscapeMask0=ESCAPE_MASK << bitsUsed;
		final int shiftedEscapeMask1=shiftedEscapeMask0 << 8;
		final int shiftedEscapeMask2=shiftedEscapeMask1 << 8;
		final int shiftedEscapeMask3=shiftedEscapeMask2 << 8;
		
		sourceBuffer.limit(end);
		while (sourceBuffer.remaining() >= 4 ) {
			sourceLong = sourceLong << 32 & (sourceBuffer.getInt() & LOWER_BITS_MASK);
			if ((sourceLong & ESCAPE_MASK3) == 0 && sourceBuffer.remaining() > 0) {
				sourceLong=(sourceLong & 0xffff00000000l) | ((sourceLong & 0xffffffl) << 8) | (sourceBuffer.get() & 0xffl);
			} else if ((sourceLong & ESCAPE_MASK2) == 0 && sourceBuffer.remaining() > 0) {
				sourceLong=(sourceLong & 0xffffff000000l) | ((sourceLong & 0xffffl) << 8) | (sourceBuffer.get() & 0xffl);
			} 
			//Potential for multiple escapes
			if ((sourceLong & ESCAPE_MASK1) == 0 && sourceBuffer.remaining() > 0) {
				sourceLong=(sourceLong & 0xffffffff000000l) | ((sourceLong & 0xffl) << 8) | (sourceBuffer.get() & 0xffl);
			} else if ((sourceLong & ESCAPE_MASK0) == 0 && sourceBuffer.remaining() > 0) {
				sourceLong=(sourceLong & 0xffffffffff000000l) | (sourceLong << 8) | (sourceBuffer.get() & 0xffl);
			}
			myLong = (myLong << 32) | (sourceLong & LOWER_BITS_MASK);
			if ((myLong & shiftedEscapeMask3)==0 || (myLong & shiftedEscapeMask2)==0 || (myLong & shiftedEscapeMask2)==1 || (myLong & shiftedEscapeMask0)==0) {
			}
		}
	}*/
	
	public void appendEscaped(BitStream source, int end) {
		buffer.limit(buffer.capacity());
		byteAlignSource(source);
		int sourceInt=ALL_BITS_MASK;
		final ByteBuffer sourceBuffer=source.buffer;
		final int shift=8-bitsUsed;
		
		int myInt=(ALL_BITS_MASK << bitsUsed) | ((buffer.get(buffer.position())&0xff)>>>shift);

		//Bottom bits mask
		final int bottomMask=((1<<(8+bitsUsed))-1);
		final int topMask=~bottomMask;
		final int shiftedEscapeMask=ESCAPE_MASK << bitsUsed;
		final int shiftedEscape=(ESCAPE_BYTE & 0xff) << (8+bitsUsed);
		final int myBitsUsed=bitsUsed;
		//Create local references for faster execution
		final ByteBuffer myBuffer=buffer;
		
		while (sourceBuffer.position() < end) {
			int sourceByte=sourceBuffer.get() & 0xff;
			sourceInt = (sourceInt << 8) | sourceByte;
			if ((sourceInt & ESCAPE_MASK) == 0 && sourceBuffer.remaining() > 0) {
				int nextByte=sourceBuffer.get(sourceBuffer.position()) & 0xff;
				if (nextByte < 4) {
					sourceBuffer.get();//Advance the pointer
					sourceInt =(sourceInt <<8 ) | nextByte;
					sourceByte=nextByte;
				}
			}
			
			myInt=(myInt << 8) | sourceByte;
			if ((myInt & shiftedEscapeMask)==0) {
				//Take the top bits and shift them up 8 to make room for the escape
				//Insert the pre-shifted escape byte
				//Keep the bottom bits;
				myInt = ((myInt & topMask) << 8) | shiftedEscape | (myInt & bottomMask);
				myBuffer.put(ESCAPE_BYTE);
			}
			myBuffer.put((byte)(myInt>>>myBitsUsed));
		}
		bitsUsed=0;//The shifting we are doing essentially zeros this out
		appendBits(myBitsUsed, myInt & ((1<<myBitsUsed)-1));
		if (bitsUsed != 0) {
			myBuffer.get();//Advance One byte;
			bitsUsed=0;
		}
		buffer.limit(myBuffer.position());
	}
	/*private long appendEscaped(long myLong, int bytes) {
		long shiftedEscapeMask=ESCAPE_MASK1 << (bitsUsed + 8*bytes);
		int writeBytes=bytes;
		
		while (bytes > 0) {
			if ((myLong & shiftedEscapeMask)==0) {
				final long bottomMask=((1<<(8*bytes+bitsUsed))-1);
				final long topMask=~bottomMask;
				final long shiftedEscape=ESCAPE_LONG << (8*bytes+bitsUsed);

				myLong = ((myLong & topMask) << 8) | shiftedEscape | (myLong & bottomMask);
				writeBytes++;
			}
			bytes--;
		}
		long writeMask=0xffl << (8*(writeBytes-1) + bitsUsed);
		for (int i=0;i<writeBytes;i++) {
			buffer.put((byte)((myLong))
		}
		return myLong;
	}*/
	public void fillByte() {
		if (bitsUsed < 8) {
			int remaining=8-bitsUsed;
			int bits=((1<<remaining)-1);
			setBits(remaining, bits);
		}
	}
	public int getExpGolombCode() {
		int bits=0;
		while (getBit()==0) {
			bits++;
		}
		if (bits > 0) {
			return (1 << bits) - 1 + getBits(bits);
		} else {
			return 0;
		}
	}
	public void setExpGolombCode(int v) {
		int mask=2;
		int zeros=0;
		while (v > mask-2) {
			zeros++;
			mask <<=1;
		}
		int bits=zeros*2+1;
		mask>>=1;
		int value=mask + ((mask-1) & (v+1));
		setBits(bits,value);
	}
	/**
	 * Move to the next whole byte; 
	 */
	public void byteAlign() {
		if (bitsUsed!=0) {
			buffer.get();
			bitsUsed=0;
		}
	}
	public static void main(String args[]) {
		BitStream bitStream=new BitStream(new byte[100]);
		//bitStream.testCopy();
		//bitStream.setPosition(0);
		//System.out.println(bitStream.getBits(3));
		//System.out.println(bitStream.getBits(5));
		//System.out.println(bitStream.getBits(15));
		//System.out.println(bitStream.getBits(2));
		//System.out.println(bitStream.getBits(15));
		//bitStream.getBits(1);
		//bitStream.setBits(4, 15);
		//System.out.println(bitStream.buffer.get(0));
		
		//System.out.println((int)0xffffffff);
		for (int i=0;i<10000;i++) {
			bitStream.setPosition(0);
			bitStream.setExpGolombCode(i);
			bitStream.setPosition(0);
			if (i != bitStream.getExpGolombCode()) {
				System.out.println(i);
			}
		}
	}
	private void byteAlignSource(BitStream source) {
		if (source.bitsUsed > 0) {
			//byte align the source
			int bitsRemaining=8-source.bitsUsed;
			this.appendBits(bitsRemaining, source.getBits(bitsRemaining));
		}
	}
	public void appendFrom(BitStream inBitStream) {
		buffer.limit(buffer.capacity());
		byteAlignSource(inBitStream);
		if (bitsUsed!=0) {
			ByteBuffer inByteBuffer=inBitStream.getByteBuffer();
			while (inByteBuffer.remaining() > 0) {
				this.appendBits(8, inByteBuffer.get() & 0xff);//TODO: Optimize
			}
			buffer.limit(buffer.position());
		} else {
			buffer.put(inBitStream.getByteBuffer());
			buffer.limit(buffer.position());
		}
	}
}

