package common;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

/**
 * This class was created to prevent OutOfMemory errors
 * Associated with mapping chunks of files
 * Java isn't very good about releasing this memory
 * unless you issue a System.gc().
 *
 */
public class DirectMemoryManager {
	public static final long MAX_ALLOCATION=192*1024*1024;
	private static long memoryAllocated=0;
	
	public static MappedByteBuffer mapMemory(FileChannel fileChannel, FileChannel.MapMode mapMode, long startPosition, long bytesToMap) throws IOException {
		MappedByteBuffer buffer;
		try {
			buffer = fileChannel.map(mapMode, startPosition, bytesToMap);
		} catch (Throwable e) {
			gc();
			buffer = fileChannel.map(mapMode, startPosition, bytesToMap);
		}
		memoryAllocated += bytesToMap;
		if (memoryAllocated > MAX_ALLOCATION) {
			gc();
		}
		return buffer;
	}
	
	public static ByteBuffer allocateDirect(int bytes) {
		ByteBuffer buffer;
		try {
			buffer = ByteBuffer.allocateDirect(bytes);
		} catch (OutOfMemoryError e) {
			gc();
			buffer = ByteBuffer.allocateDirect(bytes);
		}
		memoryAllocated += bytes;
		if (memoryAllocated > MAX_ALLOCATION) {
			gc();
		}
		return buffer;
		
	}
	private static void gc() {
		System.gc();
		memoryAllocated=0;
	}
}
