package common;

import java.io.IOException;
import java.util.ArrayList;

public class ExternalProcess {
	private Logger logger=Logger.getLogger(this.getClass());
	private String progressMask;
	private String filterMask;
	private Process process;
	private Job job;
	private InputStreamHandler logHandler;
	
	public ExternalProcess(Job job, String progressMask, String filterMask, InputStreamHandler logHandler) {
		this.progressMask=progressMask;
		this.filterMask=filterMask;
		this.job=job;
		this.logHandler=logHandler;
	}
	public static String unsplit(String command[]) {
		StringBuffer sb=new StringBuffer();
		for (int i=0;i<command.length;i++) {
			sb.append(command[i]);
			sb.append(' ');
		}
		return sb.toString();
	}	
	public int start(String command[]) throws IOException {
		//job.setSize(100);
		job.reset();
		job.setStatus(Job.STATUS_RUNNING);
		job.setPosition(0);
		process=Runtime.getRuntime().exec(command, null, null);
		String commandString=unsplit(command);
		logger.info(commandString);
		int exitValue=Integer.MIN_VALUE;
		while (exitValue==Integer.MIN_VALUE) {
			try {
				exitValue=process.exitValue();
			} catch (IllegalThreadStateException e) {
			}
			ArrayList <String> newList=logHandler.logStream(process.getInputStream());
			if (!newList.isEmpty()) {
				processStatus(newList);
				logHandler.commit(newList);
			}
			newList=logHandler.logStream(process.getErrorStream());
			if (!newList.isEmpty()) {
				processStatus(newList);
				logHandler.commit(newList);
			}
			try {
				Thread.sleep(100);
				//System.out.println("Sleeping");
			} catch (InterruptedException e) {
			}
		}
		logHandler.flush();
		process=null;
		job.done();
		return exitValue; 
		
	}
	private void processStatus(ArrayList<String> newList) {
		boolean scan=true;
		for (int i=newList.size()-1;i>=0;i--) {
			String myString=newList.get(i);
			if (filterMask != null) {
				int pos=myString.indexOf(filterMask);
				if (pos>=0) {
					if (logger.isDebug()) {
						logger.debug("Filter: " + myString);
					}
					newList.remove(i);
					continue;
				}
			}
			int pos=myString.indexOf(progressMask);
			if (pos>=0) {
				String line=newList.remove(i);
				String percentString=line;
				if (scan) {
					scan=false;  //Since we are walking backward, only take the last status.
					if (pos==0) {
						percentString=percentString.substring(progressMask.length());
					} else {
						percentString=percentString.substring(0, pos);
					}
					if (percentString.charAt(percentString.length()-1)=='%') {
						percentString=percentString.substring(0, percentString.length()-1);
					}
					try {
						int percent=(int)Float.parseFloat(percentString);
						job.setPosition(job.getSize()/100*percent);
						if (logger.isDebug()) {
							logger.debug("Status: " + line + " (" + percentString +") " + percent);
						}
					} catch (NumberFormatException nfe) {
						//Do nothing
						logger.debug("Error:  " + nfe);
					}
				}
			}
		}
	}
	public void abort() {
		process.destroy();
	}
}
