package common;

import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * @author Dustin Burckhardt
 *
 */
public interface JobStreamI {
	
	/**
	 * Sets a minimum buffer size
	 * @param bytes
	 * @return
	 */
	public void setChunkSize(int bytes);
	
	/**
	 * A round about way of suggesting a buffer size
	 * @param chunkSize
	 * @param numberOfChunks
	 */
	public void setBufferChunks(int chunkSize, int numberOfChunks);

	/**
	 * Looks ahead into the buffer
	 * @param offset
	 * @return
	 * @throws IOException
	 */
	public byte peek(int offset) throws IOException;
	
	//public abstract int readFully(byte buffer[], int offset, int len) throws IOException;
	//public abstract ByteBuffer readFully(int len) throws IOException;
	//public int readFully(ByteBuffer byteBuffer);
	
	/**
	 * Skips n bytes
	 */
	public int skip(int bytes) throws IOException;
	
	/**
	 * Resets the stream to the beginning (optional)
	 * @throws IOException
	 */
	public void reset() throws IOException;
	
	/**
	 * Is end of stream
	 * @return
	 * @throws IOException
	 */
	public boolean isEOS() throws IOException;
	
	public void open() throws IOException;
	
	public long getStreamSize();
	
	/**
	 * Forces and end to the stream size
	 * @param size
	 */
	public void setStreamSize(long size);
	
	public void close();
	
	public Job getJob();
	
	/**
	 * Operation that returns a byteBuffer slice from the current position
	 * @param len
	 * @return
	 */
	public ByteBuffer slice(int len) throws IOException;
}
