package common;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.WritableByteChannel;

public class MuxBuffer {
	protected ByteBuffer buffer;
	
	public MuxBuffer(int bytes) {
		
		buffer=DirectMemoryManager.allocateDirect(bytes);
		buffer.limit(0);
	}
	public MuxBuffer(ByteBuffer buffer) {
		this.buffer=buffer;
	}
	public MuxBuffer(byte aBuffer[]) {
		this.buffer=ByteBuffer.wrap(aBuffer);
	}
	final public int limit() {
		return buffer.limit();
	}
	final public int remaining() {
		return buffer.remaining();
	}
	final public byte get(int i) {
		return buffer.get(i);
	}
	final public byte get() {
		return buffer.get();
	}
	
	final public int copyTo(int myOffset, byte inBuffer[], int inOffset, int inLen) {
		buffer.position(myOffset);
		buffer.get(inBuffer, inOffset, inLen);
		//System.arraycopy(buffer, myOffset, inBuffer, inOffset, inLen);
		return inLen;
	}
	/*final public int copyTo(byte aBuffer[]) {
		return copyTo(0, aBuffer, 0, len);
	}*/
	final public int write(WritableByteChannel out, int myStart, int aLen) throws IOException {
		buffer.position(myStart);
		//int myLimit=buffer.limit();
		buffer.limit(myStart+aLen);
		out.write(buffer);
		//out.write(buffer, myStart, aLen);
		//buffer.limit(myLimit);
		return aLen;
	}
	public int write(WritableByteChannel out) throws IOException {
		//return write(out, 0, len);
		buffer.position(0);
		return out.write(buffer);
	}
	public int capacity() {
		return buffer.capacity();
	}
	public void limit(int len) {
		this.buffer.limit(len);
	}
	/*public final void appendFrom(MuxBuffer inBuffer, int inOffset, int inLen) {
		appendFrom(inBuffer.buffer, inOffset, inLen);
	}*/
	public final void appendFrom(JobStreamI stream, int inLen) throws IOException {
		extend(buffer.limit()+inLen);
		ByteBuffer inBuffer=stream.slice(inLen);
		buffer.put(inBuffer);
	}
	protected final void extend(int size) {
		if (size > buffer.capacity()) {
			grow(size);
		}
		buffer.limit(size);
	}
	public final void appendFrom(ByteBuffer inBuffer) {
		extend(this.limit() + inBuffer.limit());
		buffer.put(inBuffer);
	}
	public final void appendFrom(ByteBuffer inBuffer, int inOffset, int inLen) {
		extend(this.limit() + inLen);
		
		int inPos=inBuffer.position();
		int inLimit=inBuffer.limit();
		inBuffer.position(inOffset);
		inBuffer.limit(inOffset+inLen);
		buffer.put(inBuffer);
		inBuffer.limit(inLimit);
		inBuffer.position(inPos);
	}
	public final void appendFrom(byte inBuffer[], int inOffset, int inLen) {
		extend(buffer.limit()+inLen);
		buffer.put(inBuffer, inOffset, inLen);
	}
	public final void copyFrom(int myOffset, byte inBuffer[]) {
		buffer.position(myOffset);
		buffer.put(inBuffer);
	};
	protected void grow(int size) {
		//Add 10% so we don't grow by just a byte or two
		ByteBuffer newBuffer=DirectMemoryManager.allocateDirect((size * 11)/10);
		buffer.rewind();
		newBuffer.put(buffer);
		buffer=newBuffer;
	}
	public void discard(int bytes) {
		int limit=buffer.limit()-bytes;
		buffer.position(bytes);
		buffer.compact();
		buffer.limit(limit);
		//buffer.limit(buffer.capacity());
		//buffer.position(pos-bytes);
	}
	public long getLong(int offset, int size) {
		return Util.getLong(buffer, offset, size, false);
	}
	public void set32(long number, int offset) {
		buffer.putInt(offset, (int)number);
		//Util.set32(buffer, offset, number);
	}
	public final int get16(int offset) {
		//return Util.get16(buffer, offset);
		//return  ((buffer[offset]&0xFF)<<8) + (buffer[offset+1]&0xFF);
		return buffer.getShort(offset) & 0xffff;
	}
	public void set16(int value, int offset) {
		buffer.putShort(offset, (short)value);
	}
	public long getLong(int offset, int size, boolean littleEndian) {
		return Util.getLong(buffer, offset, size, littleEndian);
	}
	public void put(byte b) {
		extend(buffer.position()+1);
		buffer.put(b);
	}
	public void setByte(byte b, int offset) {
		buffer.put(offset, b);
	}
	public ByteBuffer getByteBuffer() {
		return buffer;
	}
	public void setByteBuffer(ByteBuffer byteBuffer) {
		buffer=byteBuffer;
		//byteBuffer.limit(0);
	}
	public ByteBuffer setReadWrite() {
		if (buffer.isReadOnly()) {
			//Prevent RO packets from blowing chunks
			ByteBuffer newBuffer=DirectMemoryManager.allocateDirect(buffer.limit());
			newBuffer.put(buffer);
			buffer=newBuffer;
		}
		return buffer;
	}
}
