package common;

import common.mpeg.MpegConstants;


public class StreamDescriptor {
	public static final int CLASSIFICATION_UNKNOWN=0;
	public static final int CLASSIFICATION_VIDEO=1;
	public static final int CLASSIFICATION_AUDIO=2;
	public static final int CLASSIFICATION_SUBTITLE=3;
	public static final int CLASSIFICATION_EAC3TO_CHAPTERS=4;
	
	public static final String CODEC_UNKNOWN="Unknown";
	
	public static final int VALUE_UNKNOWN=Integer.MIN_VALUE;
	private static final byte[] TS_DESCRIPTOR_NONE=new byte[0];
	
	private int streamId;
	protected String codec=CODEC_UNKNOWN;
	protected int classification=CLASSIFICATION_UNKNOWN;
	protected String fileNameExtension=".raw";
	protected byte tsType=MpegConstants.TS_STREAM_TYPE_INTERACTIVE_STREAM;//Least likely to be identified
	protected int frameTime=VALUE_UNKNOWN;
	protected StreamDescriptor coreStreamDescriptor;
	private int programNumber=VALUE_UNKNOWN;
	protected String language;
	
	public StreamDescriptor(int streamId) {
		this.streamId=streamId;
	}
	public StreamDescriptor(int stream, int classification) {
		this(stream);
		this.classification=classification;
	}
	
	public String getCodec() {
		return codec;
	}

	public void setCodec(String codec) {
		this.codec = codec;
	}
	
	public int hashCode() {
		return Integer.valueOf(streamId).hashCode();
	}
	
	public boolean equals(Object o) {
		if (super.equals(o)) {
			return true;
		} else if (o instanceof Integer) {
			return o.equals(Integer.valueOf(streamId));
		} else if (o instanceof StreamDescriptor) {
			return streamId==o.hashCode();
		} else {
			return false;
		}
	}
	public int getStreamId() {
		return streamId;
	}
	public String toString() {
		StringBuffer sb=new StringBuffer("0x");
		if (programNumber != VALUE_UNKNOWN) {
			sb.append(Integer.toString(programNumber, 16));
			sb.append(':');
		}
		sb.append(Long.toString(streamId, 16));

		sb.append(' ');
		if (language != null) {
			sb.append('(');
			sb.append(language);
			sb.append(") ");
		}
		sb.append(codec);
		if (coreStreamDescriptor != null) {
			sb.append(" (Core: ");
			sb.append(coreStreamDescriptor.toString());
			sb.append(')');
		}
		return sb.toString();
	}
	public boolean equals(int stream) {
		return this.streamId==stream;
	}
	
	public static int getStream(byte type, byte substream) {
		return ((type&0xFF)<<8) | (substream&0xFF);
	}
	public int getClassification() {
		return classification;
	}
	public void setClassification(int type) {
		this.classification = type;
	}
	public String getFileNameExtension() {
		return fileNameExtension;
	}
	public void setFileNameExtension(String fileNameExtension) {
		this.fileNameExtension = fileNameExtension;
	}
	public byte getTsType() {
		return tsType;
	}
	public void setTsType(byte tsType) {
		this.tsType = tsType;
	}
	public int getFrameTime() {
		return frameTime;
	}
	public void setFrameTime(int frameTime) {
		this.frameTime = frameTime;
	}
	public StreamDescriptor getCoreStreamDescriptor() {
		return coreStreamDescriptor;
	}
	public void setCoreStreamDescriptor(StreamDescriptor coreStreamDescriptor) {
		this.coreStreamDescriptor = coreStreamDescriptor;
	}
	public void setProgramNumber(short programNumber) {
		this.programNumber=programNumber & 0xffff;
	}
	public short getProgramNumber() {
		return (short)programNumber;
	}
	public String getLanguage() {
		return language;
	}
	public void setLanguage(String language) {
		this.language = language;
	}
	/**
	 * Returns the transport stream element stream descriptor
	 * 
	 * @return
	 */
	public byte[] getTsEsDescriptor() {
		if (language==null) {
			return TS_DESCRIPTOR_NONE;
		} else {
			byte[] descriptor={0xa, 0x4, 0,0,0,0};
			for (int i=0; i<Math.min(4, language.length());i++) {
				descriptor[i+2]=(byte)language.charAt(i);
			}
			return descriptor;
		}
	}
}
