package common;


public class VideoStreamDescriptor extends StreamDescriptor {
	private int width=VALUE_UNKNOWN;
	private int height=VALUE_UNKNOWN;
	private int frameRateRatio[];
	private int aspectRatio[];
	private int pixelAspectRatio[];
	private int interlaced=VALUE_UNKNOWN;
	private String profileName;
	private String levelName;
	
	public VideoStreamDescriptor(int stream) {
		super(stream);
		classification=CLASSIFICATION_VIDEO;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}
	public String toString() {
		StringBuffer sb=new StringBuffer(super.toString());
		if (profileName != null) {
			sb.append(' ');
			sb.append(profileName);
			if (levelName !=null) {
				sb.append('-');
				sb.append(levelName);
			}
		}
		int dWidth=width;
		int dHeight=height;
		if (pixelAspectRatio !=null && width != VALUE_UNKNOWN) {
			dWidth=dWidth*pixelAspectRatio[0]/pixelAspectRatio[1];
		}
		/*if (aspectRatio != null && height != VALUE_UNKNOWN) {
			dHeight-=dHeight % aspectRatio[1];//Hack to convert 1088 to 1080 so the numbers work out.
			dWidth=dHeight*aspectRatio[0]/aspectRatio[1];
		}*/
		if (dWidth != VALUE_UNKNOWN && dHeight != VALUE_UNKNOWN) {
			sb.append(' ');
			sb.append(dWidth);
			sb.append('x');
			sb.append(dHeight);
			
			if (interlaced==1) {
				sb.append('i');
			} else if (interlaced==0) {
				sb.append('p');
			}
			
			if (width != dWidth || height != dHeight) {
				sb.append(" (");
				sb.append(width);
				sb.append('x');
				sb.append(height);
				sb.append(')');
			}
		}

		if (frameRateRatio != null) {
			sb.append(" fps: ");
			sb.append(frameRateRatio[0]);
			if (frameRateRatio[1]!=1) {
				sb.append('/');
				sb.append(frameRateRatio[1]);
			}
		}
		if (aspectRatio != null) {
			sb.append(" AR: ");
			sb.append(aspectRatio[0]);
			sb.append(':');
			sb.append(aspectRatio[1]);
		}
		return sb.toString();
	}

	public int[] getFrameRateRatio() {
		return frameRateRatio;
	}

	public void setFrameRateRatio(int[] frameRateRatio) {
		this.frameRateRatio = frameRateRatio;
	}

	public int[] getAspectRatio() {
		return aspectRatio;
	}

	public void setAspectRatio(int[] aspectRatio) {
		this.aspectRatio = aspectRatio;
	}

	public int getInterlaced() {
		return interlaced;
	}

	public void setInterlaced(int interlaced) {
		this.interlaced = interlaced;
	}

	public int[] getPixelAspectRatio() {
		return pixelAspectRatio;
	}

	public void setPixelAspectRatio(int[] pixelAspectRatio) {
		this.pixelAspectRatio = pixelAspectRatio;
	}

	public String getProfileName() {
		return profileName;
	}

	public void setProfileName(String profileName) {
		this.profileName = profileName;
	}

	public String getLevelName() {
		return levelName;
	}

	public void setLevelName(String levelName) {
		this.levelName = levelName;
	}
}
