package muxer;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import common.StreamDescriptor;
import common.packet.ClockReferenceI;
import common.packet.Packet;

public class MuxUtil {
	public static long calculateDelta(Set longSet) {
		HashMap deltaMap=new HashMap();
		
		long sum=0;
		long mostFrequent=-1;
		Integer mostFrequentCount=Integer.valueOf(0);
		long previous=-1;
		for (Iterator i=longSet.iterator();i.hasNext();) {
			long current=((Long)i.next()).longValue();
			if (previous != -1) {
				long delta=current - previous;
				sum+=delta;
				Integer count=(Integer)deltaMap.get(Long.valueOf(delta));
				if (count==null) {
					count=Integer.valueOf(0);
				}
				count=Integer.valueOf(count.intValue()+1);
				deltaMap.put(Long.valueOf(delta), count);
				if (count.compareTo(mostFrequentCount) > 0) {
					mostFrequentCount=count;
					mostFrequent=delta;
				}
			}
			previous=current;
		}
		//If the offsets are fairly evenly distributed, return the average, otherwise return the mode
		if (deltaMap.size() > longSet.size()/4) {
			return sum/longSet.size();
		} else {
			return mostFrequent;
		}
	}
	/**
	 * @param packetCollection
	 * @param syncStreamId
	 * @param delta
	 * @return 0=the recommend stream start time, 1=the lead time at the sync stream (if video arrives before audio)
	 */
	public static long[] calculateStreamTime(Collection packetCollection, int syncStreamId, long delta) {
		long time=0;
		long offset=delta;
		for (Iterator i=packetCollection.iterator();i.hasNext();) {
			Packet packet=(Packet)i.next();
			if (packet.getStreamId()==syncStreamId && (time=packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS))!=ClockReferenceI.CLOCK_REFERENCE_NA) {
				//Unlikely
				if (time==ClockReferenceI.CLOCK_REFERENCE_NONE) {
					offset+=delta;
				} else {
					time -=offset;
					break;
				}
			}
		}
		int syncCount=0;
		long leadTime=0;
		for (Iterator i=packetCollection.iterator();i.hasNext();) {
			Packet packet=(Packet)i.next();
			long cr=packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS);
			if (cr!=ClockReferenceI.CLOCK_REFERENCE_NA) {
				if (packet.getStreamId()==syncStreamId) {
					time+=delta;
					syncCount++;
					if (cr!=ClockReferenceI.CLOCK_REFERENCE_NONE) {
						leadTime=time-cr;
					}
				}
				if (cr!=ClockReferenceI.CLOCK_REFERENCE_NONE && cr < time) {
					time=cr;
				}
			}
		}
		time-=syncCount * delta;
		return new long[] {time, leadTime};
	}
	public static long getMode(Iterator pi, int streamId) {
		long lastCr=Long.MIN_VALUE; 
		HashMap deltaMap=new HashMap();
		
		while (pi.hasNext()) {
			long cr;
			Packet packet=(Packet)pi.next();
			if (packet.getStreamId()==streamId && (cr=packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS))!=ClockReferenceI.CLOCK_REFERENCE_NA) {
				if (lastCr!=Long.MIN_VALUE) {
					Long delta=Long.valueOf(cr-lastCr);
					int count[]=(int[])deltaMap.get(delta);
					if (count==null) {
						deltaMap.put(delta,count=new int[] {0});
					}
					count[0]+=1;
				}
				lastCr=cr;
			}
		}
		int max=0;
		long delta=0;
		for (Iterator i=deltaMap.keySet().iterator();i.hasNext();) {
			Long key=(Long)i.next();
			int count[]=(int[])deltaMap.get(key);
			if (count[0]>max) {
				delta=key.longValue();
				max=count[0];
			} 
		}
		return delta;
	}
	public static long getNextPacketPts(int streamId, PacketBuffer packetBuffer) {
		for (Iterator i=packetBuffer.iterator();i.hasNext();) {
			Packet packet=(Packet)i.next();
			if (packet.getStreamId()==streamId) {
				long pts=packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS);
				if (pts >=0) {
					return pts;
				}
			}
		}
		return Long.MIN_VALUE;
	}
	/**
	 * Attempts to find a constant rate stream for muxing purposes (calculating arrival, SCR etc.)
	 * @param streamDescriptorMap
	 * @return
	 */
	public static StreamDescriptor getLinearStream(Map<Integer, StreamDescriptor> streamDescriptorMap) {
		return getLinearStream(streamDescriptorMap.values());
	}
	public static StreamDescriptor getLinearStream(Collection<StreamDescriptor>coll) {
		for (StreamDescriptor sd : coll) {
			if (sd.getClassification()==StreamDescriptor.CLASSIFICATION_AUDIO) {
				return sd;
			}
		}
		return null;
	}
}
