package muxer.filter;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Iterator;

import muxer.MuxFilter;
import muxer.MuxManager;

import common.MuxBuffer;
import common.StatusI;
import common.packet.Packet;
import common.packet.PacketPart;
import common.packet.PesPacket;
import common.packet.PesPacketPart;

import demux.ts.TsInfo;
import demux.ts.TsRawPacketPart;

/**
 * Converts M2ts&TsRawPackets to PesPackets
 *
 */
public class TsRawToPesMuxFilter extends MuxFilter {
	private int counter;
	private HashMap bufferMap=new HashMap();
	
	private class IndexedBuffer extends MuxBuffer {
		private int index;
		private int pid;
		
		//public IndexedBuffer(int len, int pid, int index) {
		public IndexedBuffer(int len, int pid) {
			super (len);
			this.pid=pid;
			//this.index=counter++;
		}
	}
	
	public int addPacket(Packet packet) throws IOException {
		int rc=StatusI.CONTINUE;
		TsRawPacketPart packetPart=(TsRawPacketPart)packet.getPayload();
		if (packetPart.isAvStream()) {
			TsInfo info=packetPart.getTsInfo();
			IndexedBuffer buffer=(IndexedBuffer)bufferMap.get(Integer.valueOf(info.pid));
			if (buffer==null) {
				if (info.pidStart) {
					bufferMap.put(Integer.valueOf(info.pid), buffer=new IndexedBuffer(packetPart.limit(), info.pid /*, packet.getIndex()*/));
					buffer.limit(0);
				} else {
					return rc;
				}
			}
			if (info.pidStart) {
				rc=flushPacket(buffer, muxManager);
				buffer.limit(0);
				buffer.index=counter++;
			}
			int offset=packetPart.getTsSyncOffset()+4;
			if (info.adaptationControl >= 0x20) {
				offset += (packetPart.get(offset) & 0xff) + 1; 
			}
			buffer.appendFrom(packetPart.getByteBuffer(), offset, packetPart.limit()-offset);
		} 
		return rc;
	}
	
	private int flushPacket(IndexedBuffer buffer, MuxManager muxManager) throws IOException{
		if (buffer.limit() > 0) {
			PesPacket pesPacket=new PesPacket(buffer.pid, buffer.index);
			int headerLen=(buffer.get(8) & 0xff) + 9;
			PesPacketPart header=muxManager.getPesPacketPart(headerLen);
			ByteBuffer byteBuffer = buffer.getByteBuffer();
			header.appendFrom(byteBuffer, 0, headerLen);
			PacketPart payload=muxManager.getPacketPart(buffer.limit() - headerLen);
			payload.appendFrom(byteBuffer, headerLen, buffer.limit()-headerLen);
			pesPacket.setHeader(header);
			pesPacket.setPayload(payload);
			return super.addPacket(pesPacket);
		} else {
			return StatusI.CONTINUE;
		}
	}
	
	protected void streamEnded(MuxManager muxManager) throws IOException {
		for (Iterator i=bufferMap.values().iterator();i.hasNext();) {
			IndexedBuffer buffer=(IndexedBuffer)i.next();
			flushPacket(buffer, muxManager);
		}
	}
	public int getPosition() {
		return POSITION_PRE_BUFFER;
	}	
}
