package ui;

import javax.swing.JProgressBar;

import common.Job;
import common.JobListenerI;
import common.StatusI;
import common.Util;

public class ProgressBarJobListener implements JobListenerI {
	private boolean cancel;
	private long lastTime=0;
	private int lastPosition=0;
	private int lastStatus=Job.STATUS_WAITING;
	
	private JProgressBar progressBar;
	
	public ProgressBarJobListener(JProgressBar progressBar) {
		this.progressBar=progressBar;
	}
	
	private void reset() {
		cancel=false;
		lastTime=System.currentTimeMillis();
		lastPosition=0;
	}
	public int jobStatus(Job job) {
		int status=job.getStatus();
		String statusText=null;
		
		if (status==Job.STATUS_RUNNING && lastStatus != Job.STATUS_RUNNING) {
			reset();
			statusText=job.getDescription();
			//System.out.println("Reset()");
		}
		//This is here in case the job changes the size (for example a scan)
		progressBar.setMaximum(job.getSize());
		int position=job.getPosition();
		//progressBar.setString(job.getDescription());
		progressBar.setValue(position);
		
		long now=System.currentTimeMillis();
		long mbps=0,eta=0;
		
		lastStatus=status;
		
		if (status==Job.STATUS_COMPLETE) {
			statusText="Completed " + job.getDescription() + " in " + Util.timeToString(now-job.getStartTime());
			progressBar.setValue(job.getSize());
		} else if (status==Job.STATUS_FAILED || status==Job.STATUS_ABORTED){
			statusText=Job.STATUS_NAMES[status] + " " + job.getDescription();
		} else if (lastPosition==position) {
			//Do Nothing
			//This was causing the status to say 0 mB/s
		} else if (now > lastTime + 1000) {
			
			mbps=(position-lastPosition)/(now - lastTime);
			//System.out.println(position + ":" + lastPosition + ":" + (now-lastTime));
			if (mbps > 0) {
				eta=(job.getSize()-position)/mbps;
				lastPosition=position;
				lastTime=now;
			} 
			statusText=job.getDescription() + " @ " + mbps + " mB/s - ETA: " + Util.timeToString(eta);
			progressBar.setValue(position);
		}
		if (statusText != null) {
			progressBar.setString(statusText);
		}

		return cancel ? StatusI.ABORT : StatusI.CONTINUE;
	}
	
	public void cancel() {
		this.cancel=true;
	}
}
