package ui;

import io.NameTypeFileFilter;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.prefs.Preferences;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;

import muxer.MuxFilter;
import muxer.MuxManager;
import muxer.MuxManager.DemuxerMuxerInfo;
import muxer.PacketBuffer;
import muxer.filter.ContiguousTimestampMuxFilter;
import muxer.filter.CoreMuxFilter;
import muxer.filter.Vc1HdDvdToBluRayMuxFilter;

import common.FileStream;
import common.Job;
import common.JobStreamI;
import common.Logger;
import common.LoggerAppenderI;
import common.StreamDescriptor;
import common.StreamIdentifier;
import common.VersionI;
import common.VideoStreamDescriptor;
import common.ts.ProgramAssociationTable;

import demux.Demuxer;
import demux.microsoft.WtvFileStream;

public class RemuxToolUI extends JFrame implements VersionI {
	private static final String LIST_BOX_PROTOTYPE="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
	private static final String SAVE_AS_TEXT="Save As...";
	private static final String CANCEL_TEXT="Cancel";
	private static final int LOG_ROWS=5;
	private static final String PREFS_X="xPos";
	private static final String PREFS_Y="yPos";
	private static final String PREFS_WIDTH="width";
	private static final String PREFS_HEIGHT="height";
	private static final String PREFS_INPUT_DIR="inputDir";
	private static final String PREFS_OUTPUT_DIR="outputDir";
	
	
	private static final NameTypeFileFilter[] FILE_CHOOSERS={
		new NameTypeFileFilter(new String[]{".ty",".ty+"}, "Tivo Titles"),
		new NameTypeFileFilter(new String[]{".ts"}, "Transport Stream"),
		new NameTypeFileFilter(new String[]{".mpg"}, "MPEG-2 PES Streams"),
		new NameTypeFileFilter(new String[]{".vob"}, "DVD"),
		new NameTypeFileFilter(new String[]{".evo"}, "HD DVD"),
		new NameTypeFileFilter(new String[]{".m2ts",".mts"}, "BluRay/AVCHD Stream"),
		new NameTypeFileFilter(new String[]{".wtv"}, "Windows TV")
	};
	
	private static final HashSet<String> FILE_EXTS=new HashSet<String>();
	
	private Preferences prefs =  Preferences.userRoot().node("RemuxTool");
	private JFileChooser inputFileChooser;
	private Logger logger=Logger.getLogger(this.getClass());
	private MuxManager muxManager;
	//private FileStream fileStream;
	//private boolean cancel=false;
	
	public static void main(String args[]) {
		RemuxToolUI frame = new RemuxToolUI();
		frame.setVisible(true);
	}
	private class DemuxerMuxerNameTypeFileFilter extends NameTypeFileFilter {
		private DemuxerMuxerInfo info;
		private DemuxerMuxerNameTypeFileFilter(DemuxerMuxerInfo info) {
			super(info.getExt(), info.getName());
			this.info=info;
		}
	}
	protected RemuxToolUI (){
		this.setTitle(FULL_NAME);
		
		for (NameTypeFileFilter type : FILE_CHOOSERS) {
			for (String ext : type.getFileTypes()) {
				FILE_EXTS.add(ext);
			}
		}
		
		getContentPane().setLayout(new GridBagLayout());
		setBounds(prefs.getInt(PREFS_X,100), prefs.getInt(PREFS_Y, 100), prefs.getInt(PREFS_WIDTH, 600), prefs.getInt(PREFS_HEIGHT, 600));
		
		Insets insets=new Insets(1,1,1,1);
		
		int y=0;
		final JScrollPane inputFileNameScrollPane = new JScrollPane();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor=GridBagConstraints.NORTH;
		gbc.fill=GridBagConstraints.BOTH;
		gbc.gridwidth=2;
		gbc.weightx=1;
		gbc.gridheight=2;
		gbc.insets=insets;
		getContentPane().add(inputFileNameScrollPane, gbc);
		
		final JTextArea inputFileNameTextArea = new JTextArea();
		//inputFileNameTextArea.setRows(2);
		//inputFileNameTextArea.setEditable(false);
		inputFileNameScrollPane.setViewportView(inputFileNameTextArea);
		
		final JButton inputFileNameButton = new JButton();
		gbc = new GridBagConstraints();
		gbc.anchor=GridBagConstraints.NORTH;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx=2;
		gbc.insets=insets;
		getContentPane().add(inputFileNameButton, gbc);
		inputFileNameButton.setText("Input...");
		
		final JButton scanButton = new JButton();
		gbc = new GridBagConstraints();
		gbc.anchor=GridBagConstraints.NORTH;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx=2;
		gbc.gridy=++y;
		gbc.insets=insets;
		getContentPane().add(scanButton, gbc);
		scanButton.setText("Scan");
		
		final JLabel videoLabel=new JLabel("Video:");
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.insets=insets;
		getContentPane().add(videoLabel,gbc);
		
		final JScrollPane videoListScrollPane = new JScrollPane();
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.weightx=1;
		//gbc.weighty=.1;
		gbc.gridwidth=3;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.insets=insets;
		getContentPane().add(videoListScrollPane, gbc);
		
		final JList videoListBox = new JList();
		videoListBox.setPrototypeCellValue(LIST_BOX_PROTOTYPE);
		videoListBox.setVisibleRowCount(2);
		videoListScrollPane.setViewportView(videoListBox);
		
		final JLabel audioLabel=new JLabel("Audio:");
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.insets=insets;
		getContentPane().add(audioLabel,gbc);
		
		final JScrollPane audioListScrollPane = new JScrollPane();
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		//gbc.weighty=.2;
		gbc.weightx=1;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridwidth=3;
		gbc.insets=insets;
		getContentPane().add(audioListScrollPane, gbc);
		
		final JList audioListBox = new JList();
		audioListBox.setVisibleRowCount(3);
		audioListBox.setPrototypeCellValue(LIST_BOX_PROTOTYPE);
		audioListScrollPane.setViewportView(audioListBox);
		
		final JLabel subtitleLabel=new JLabel("Subtitles:");
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.insets=insets;
		getContentPane().add(subtitleLabel,gbc);
		
		final JScrollPane subtitleListScrollPane = new JScrollPane();
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		//gbc.weighty=.2;
		//gbc.weightx=;
		gbc.gridwidth=3;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.insets=insets;
		getContentPane().add(subtitleListScrollPane, gbc);
		
		final JList subtitleListBox = new JList();
		subtitleListBox.setPrototypeCellValue(LIST_BOX_PROTOTYPE);
		subtitleListBox.setVisibleRowCount(3);
		subtitleListScrollPane.setViewportView(subtitleListBox);
		
		final JLabel otherLabel=new JLabel("Other:");
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.insets=insets;
		getContentPane().add(otherLabel,gbc);
		
		final JScrollPane otherListScrollPane = new JScrollPane();
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		//gbc.weighty=.1;
		gbc.weightx=1;
		gbc.gridwidth=3;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.insets=insets;
		getContentPane().add(otherListScrollPane, gbc);

		final JList otherListBox = new JList();
		otherListBox.setVisibleRowCount(1);
		otherListBox.setPrototypeCellValue(LIST_BOX_PROTOTYPE);
		otherListScrollPane.setViewportView(otherListBox);
		
		final JCheckBox coreCheckBox=new JCheckBox("Core dts-HD and Dolby TrueHD", true);
		coreCheckBox.setEnabled(false);
		
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		getContentPane().add(coreCheckBox,gbc);
		
		final JCheckBox contiguousTimestampCheckBox=new JCheckBox("Ensure contiguous timestamps", false);
		contiguousTimestampCheckBox.setEnabled(false);
		
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		getContentPane().add(contiguousTimestampCheckBox,gbc);
		
		
		final JCheckBox hdDvdVc1ToBluRayCheckBox=new JCheckBox("Convert HD DVD VC1 to BluRay Compliant", false);
		hdDvdVc1ToBluRayCheckBox.setEnabled(false);
		
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		getContentPane().add(hdDvdVc1ToBluRayCheckBox,gbc);
		
		final JTextArea saveAsTextArea = new JTextArea();
		saveAsTextArea.setEditable(false);
		
		final JScrollPane outputFileNameScrollPane = new JScrollPane();
		gbc = new GridBagConstraints();
		gbc.fill=GridBagConstraints.BOTH;
		gbc.gridy=++y;
		gbc.gridwidth=2;
		gbc.weightx=1;
		//gbc.weighty=.1;
		gbc.insets=insets;
		getContentPane().add(outputFileNameScrollPane, gbc);
		outputFileNameScrollPane.setViewportView(saveAsTextArea);
		
		final JButton saveAsButton = new JButton(SAVE_AS_TEXT);
		saveAsButton.setEnabled(false);
		gbc = new GridBagConstraints();
		gbc.anchor=GridBagConstraints.NORTH;
		gbc.gridx=1;
		gbc.gridy=y;
		gbc.gridx=2;
		gbc.insets=insets;
		getContentPane().add(saveAsButton, gbc);
		
		final JProgressBar progressBar=new JProgressBar();
		progressBar.setStringPainted(true);
		progressBar.setString("Status");
		gbc = new GridBagConstraints();
//		gbc.weighty=.1;
		gbc.gridy=++y;
		gbc.weightx=1;
		gbc.gridwidth=3;
		gbc.fill=GridBagConstraints.BOTH;
		gbc.insets=insets;
		getContentPane().add(progressBar, gbc);
		
		final JLabel logLabel=new JLabel("Log:");
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.anchor=GridBagConstraints.WEST;
		gbc.insets=insets;
		getContentPane().add(logLabel,gbc);
		
		final JScrollPane logScrollPane = new JScrollPane();
		gbc = new GridBagConstraints();
		gbc.gridy=++y;
		gbc.fill=GridBagConstraints.BOTH;
		gbc.gridwidth=3;
		gbc.weightx=1;
		gbc.anchor=GridBagConstraints.SOUTH;
		gbc.weighty=1;
		gbc.insets=insets;
		getContentPane().add(logScrollPane, gbc);
		
		final JTextArea logTextArea = new JTextArea();
		logTextArea.setEditable(false);
		logTextArea.setRows(LOG_ROWS);
		logScrollPane.setViewportView(logTextArea);
		
		final ProgressBarJobListener remuxListener=new ProgressBarJobListener(progressBar);

		videoListBox.setModel(new DefaultListModel());
		audioListBox.setModel(new DefaultListModel());
		subtitleListBox.setModel(new DefaultListModel());
		otherListBox.setModel(new DefaultListModel());
		
		final HashMap<Integer, JList> listBoxMap=new HashMap<Integer, JList>();
		listBoxMap.put(StreamDescriptor.CLASSIFICATION_AUDIO, audioListBox);
		listBoxMap.put(StreamDescriptor.CLASSIFICATION_VIDEO, videoListBox);
		listBoxMap.put(StreamDescriptor.CLASSIFICATION_SUBTITLE, subtitleListBox);
		listBoxMap.put(StreamDescriptor.CLASSIFICATION_UNKNOWN, otherListBox);
		
		final ActionListener scanActionListener=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final File files[]=FileStream.getFiles(inputFileNameTextArea.getText());
				final JobStreamI fileStream=getJobStream(new Job(remuxListener), files);
				contiguousTimestampCheckBox.setEnabled(files.length > 1);
				//inputFileNameTextArea.setEditable(files.length > 1);
				
				Demuxer demuxer=MuxManager.getStreamIdentiferDemuxer(files[0]);
				muxManager=new MuxManager(demuxer);
				final DefaultListModel otherListModel=(DefaultListModel)otherListBox.getModel();
				((DefaultListModel)videoListBox.getModel()).clear();
				((DefaultListModel)audioListBox.getModel()).clear();
				((DefaultListModel)subtitleListBox.getModel()).clear();
				otherListModel.clear();

				Thread scanThread=new Thread() {
					public void run() {
						Map streamDescriptorMap;
						try {
							logger.info("Scanning " + files[0] + " for streams");
							streamDescriptorMap = muxManager.getStreamDescriptorMap(fileStream);
							//logger.info(streamDescriptorMap);
							for (Iterator i=streamDescriptorMap.values().iterator();i.hasNext();) {
								StreamDescriptor streamDescriptor=(StreamDescriptor)i.next();
								JList listBox=(JList)listBoxMap.get(Integer.valueOf(streamDescriptor.getClassification()));
								DefaultListModel model=(DefaultListModel)listBox.getModel();
								model.addElement(streamDescriptor);
							}
							videoListBox.setSelectedIndex(0);
							audioListBox.setSelectedIndex(0);
							
							for (int i=0;i<otherListModel.getSize();i++) {
								StreamDescriptor streamDescriptor=(StreamDescriptor)otherListModel.get(i);
								if (streamDescriptor.getCodec()==ProgramAssociationTable.NETWORK_INFORMATION) {
									otherListBox.setSelectedIndex(i);
								}
							}
							//Clear the buffers
							muxManager.streamEnded();
							saveAsButton.setEnabled(true);

						} catch (IOException e1) {
							e1.printStackTrace();
							logger.error(e1);
						}
					}
				};
				scanThread.start();
			}
		};
		scanButton.addActionListener(scanActionListener);
		class ScanUtil {
			private void scanFiles(File files[]) {
				StringBuffer sb=new StringBuffer();
				String sep="";
				for (int i=0;i<files.length;i++) {
					sb.append(sep);
					sb.append(files[i].getAbsolutePath());
					sep="\n";
				}
				inputFileNameTextArea.setText(sb.toString());
				scanActionListener.actionPerformed(null);
			}
		}
		final ScanUtil scanUtil=new ScanUtil();
		
		new DropTarget(inputFileNameTextArea, DnDConstants.ACTION_COPY, new DropTargetAdapter() {
			public void dragEnter(DropTargetDragEvent dtde) { 	
				try {
			        Transferable t = dtde.getTransferable();
			        if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
			            List<File> list = (List<File>)t.getTransferData(DataFlavor.javaFileListFlavor);
			        	for (File file : list) {
			        		String name=file.getName();
			        		String ext=name.substring(name.lastIndexOf('.')).toLowerCase();
			        		if (!FILE_EXTS.contains(ext)) {
						        dtde.rejectDrag();
						        return;
			        		}
			        	}
		        		return;
			        }
			    } catch (IOException e) {
			    } catch (UnsupportedFlavorException e) {
			    }
		        dtde.rejectDrag();
			}
			
			@Override
			public void drop(DropTargetDropEvent dtde) {
			    try {
			        Transferable t = dtde.getTransferable();
			        if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
			        //if (t.isDataFlavorSupported(DataFlavor.stringFlavor)) {
			        	dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
			            List<File> list = (List<File>)t.getTransferData(DataFlavor.javaFileListFlavor);
			            dtde.getDropTargetContext().dropComplete(true);
			            File[] files=new File[list.size()];
			            list.toArray(files);
			            scanUtil.scanFiles(files);
			        } else {
			        	dtde.rejectDrop();
			        }
			    } catch (IOException e) {
			    	dtde.rejectDrop();
			    } catch (UnsupportedFlavorException e) {
			    	dtde.rejectDrop();
			    }
				
			}
		});
		
		final ActionListener inputActionListener=new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				if (inputFileChooser==null) {
					String pathName=prefs.get(PREFS_INPUT_DIR, null);
					inputFileChooser=new JFileChooser(pathName);
					for (NameTypeFileFilter chooser : FILE_CHOOSERS) {
						inputFileChooser.addChoosableFileFilter(chooser);
					}
					//convertFileChooser.setAcceptAllFileFilterUsed(false);
					inputFileChooser.setMultiSelectionEnabled(true);
				}
				int returnVal = inputFileChooser.showOpenDialog(RemuxToolUI.this);
				if (returnVal==JFileChooser.APPROVE_OPTION) {
					File path=inputFileChooser.getSelectedFile().getParentFile();
					prefs.put(PREFS_INPUT_DIR, path.getAbsolutePath());
					scanUtil.scanFiles(inputFileChooser.getSelectedFiles());
				}
			}
		};
		inputFileNameButton.addActionListener(inputActionListener);

		final ActionListener saveAsListener=new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if (saveAsButton.getText()==CANCEL_TEXT) {
					remuxListener.cancel();
					return;
				}
				String pathName=prefs.get(PREFS_OUTPUT_DIR, null);
				JFileChooser saveAsFileChooser=new JFileChooser(pathName);
				saveAsFileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
				saveAsFileChooser.setAcceptAllFileFilterUsed(false);
				final File[] files=FileStream.getFiles(inputFileNameTextArea.getText());
				final JobStreamI fileStream=getJobStream(new Job(remuxListener), files);
				DemuxerMuxerInfo info[] = MuxManager.getDemuxerRemuxerInfo(files[0]);
				
				DemuxerMuxerNameTypeFileFilter firstFilter=null;
				for (int i=0;i<info.length;i++) {
					DemuxerMuxerNameTypeFileFilter fileFilter=new DemuxerMuxerNameTypeFileFilter(info[i]);
					saveAsFileChooser.addChoosableFileFilter(fileFilter);
					if (i==0) {
						firstFilter=fileFilter;
					}
				}
				saveAsFileChooser.setFileFilter(firstFilter);//Bug work around
				
				String fileName=files[0].getName();
				fileName=fileName.substring(0, fileName.lastIndexOf('.'));
				saveAsFileChooser.setSelectedFile(new File(fileName));
				int returnVal = saveAsFileChooser.showSaveDialog(RemuxToolUI.this);
				if (returnVal==JFileChooser.APPROVE_OPTION) {
					DemuxerMuxerNameTypeFileFilter fileFilter=(DemuxerMuxerNameTypeFileFilter)saveAsFileChooser.getFileFilter();
					File file=saveAsFileChooser.getSelectedFile();
					prefs.put(PREFS_OUTPUT_DIR, file.getParent());
					fileName=file.getAbsolutePath();
					if (!fileName.toLowerCase().endsWith(fileFilter.info.getExt())) {
						file=new File(fileName + fileFilter.info.getExt());
					}
					saveAsTextArea.setText(file.getAbsolutePath());
					//Setup Demuxer
					if (muxManager.getDemuxer().getClass() != fileFilter.info.getDemuxerClass()) {
						muxManager.setDemuxer(MuxManager.getDemuxer(fileFilter.info.getDemuxerClass()));
					}
					
					ArrayList streamList=new ArrayList();
					for (Iterator lbi=listBoxMap.values().iterator();lbi.hasNext();) {
						JList listBox=(JList)lbi.next();
						Object objects[]=listBox.getSelectedValues();
						for (int i=0;i<objects.length;i++) {
							streamList.add(objects[i]);
						}
					}
					muxManager.getDemuxer().setStreams(streamList);
					if (coreCheckBox.isEnabled() && coreCheckBox.isSelected()) {
						Object selected[]=audioListBox.getSelectedValues();
						for (int i=0;i<selected.length;i++) {
							StreamDescriptor streamDescriptor=(StreamDescriptor)selected[i];
							StreamDescriptor coreStreamDescriptor=streamDescriptor.getCoreStreamDescriptor();
							if (coreStreamDescriptor !=null) {
								try {
									muxManager.add(new CoreMuxFilter(streamDescriptor.getStreamId(), coreStreamDescriptor.getStreamId()));
									//Set the streamDescriptor TsType to the type of the core.  
									//This will be picked up by the PMT generator later
									streamDescriptor.setTsType(coreStreamDescriptor.getTsType());
								} catch (IOException e) {
									//Can't happen
								}
							}
						}
					}
					if (contiguousTimestampCheckBox.isSelected() && files.length > 0) {
						try {
							muxManager.add(new ContiguousTimestampMuxFilter());
						} catch (IOException e) {
							// Can't happen
							e.printStackTrace();
						}
					}
					if (hdDvdVc1ToBluRayCheckBox.isEnabled() && hdDvdVc1ToBluRayCheckBox.isSelected()) {
						for (int i=0;i<streamList.size();i++) {
							StreamDescriptor sd=(StreamDescriptor)streamList.get(i);
							if (sd.getCodec().startsWith(StreamIdentifier.VC1VideoAnalyzer.CODEC_NAME)) {
								VideoStreamDescriptor vsd=(VideoStreamDescriptor)sd;
								if (vsd.getInterlaced()==1) {
									try {
										muxManager.add(new Vc1HdDvdToBluRayMuxFilter(vsd.getStreamId()));
									} catch (IOException e) {
										// Can't happen
										e.printStackTrace();
									}
								}
								
							}
						}
					}

					MuxFilter muxer;
					try {
						muxer=MuxManager.createRemuxer(fileFilter.info.getRemuxerClass(), file);
					} catch (IOException ioe) {
						logger.error(ioe);
						return;
					}
					try {
						muxManager.add(muxer);

						Thread jobThread=new Thread() {
							public void run() {
								try {
									saveAsButton.setText(CANCEL_TEXT);
									muxManager.mux(fileStream);
								} catch (Exception ioe) {
									logger.error(ioe);
									ioe.printStackTrace();
								} finally {
									saveAsButton.setText(SAVE_AS_TEXT);
									PacketBuffer packetBuffer=muxManager.getPacketBuffer();
									if (packetBuffer != null) {
										packetBuffer.reset();
									}
								}
								fileStream.close();
							}
						};
						jobThread.start();
					} catch (Exception e) {
						e.printStackTrace();
						logger.error(e);
					}
				}
			}
		};
		saveAsButton.addActionListener(saveAsListener);
		
		final LoggerAppenderI logAppender=new LoggerAppenderI() {
			public void append(String date, String time, String loggerName, Integer level, String message) {
				//int maxLines=PropertiesReader.getInstance().getIntProperty("ui.maxLogLines",1000);
				int maxLines=1000;
				int lines=logTextArea.getLineCount();
				if (lines > maxLines) {
					int firstLine=lines-maxLines;
					try {
						int firstLinePos=logTextArea.getLineEndOffset(firstLine);
						int textLength=logTextArea.getDocument().getLength();
						String logText="... log truncated to " + maxLines + 
							" lines\n" + logTextArea.getText(firstLinePos, textLength-firstLinePos) + 
							time + ' ' + message + "\n";
						logTextArea.setText(logText);
					} catch (BadLocationException e) {
						// Should never happen
						e.printStackTrace();
					}
				} else {
					logTextArea.append(time + ' ' + message + "\n");
				}
				logTextArea.setCaretPosition(logTextArea.getDocument().getLength()-1);
				logTextArea.setRows(LOG_ROWS);
			}
		};
		Logger.addAppender(logAppender);
		
		final ActionListener remuxActionListener=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//Descriptor video[]=(Descriptor[])videoListBox.getSelectedValues();
				Object video[]=videoListBox.getSelectedValues();
				Object audio[]=audioListBox.getSelectedValues();
				Object subtitles[]=subtitleListBox.getSelectedValues();
				Object other[]=otherListBox.getSelectedValues();
				
				int pids[]=new int[video.length+audio.length+subtitles.length+other.length];
				Object all[][]={video, audio, subtitles, other};
				
			}
		};
		final ListSelectionListener audioListSelectionListener=new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				Object selected[]=audioListBox.getSelectedValues();
				boolean enableCore=false;
				for (int i=0;i<selected.length;i++) {
					StreamDescriptor streamDescriptor=(StreamDescriptor)selected[i];
					if (streamDescriptor.getCoreStreamDescriptor() != null) {
						enableCore=true;
					}
				}
				coreCheckBox.setEnabled(enableCore);
			}
		};
		audioListBox.addListSelectionListener(audioListSelectionListener);
		
		final ListSelectionListener videoListSelectionListener=new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				boolean enabled=false;
				Object selected[]=videoListBox.getSelectedValues();
				for (int i=0;i<selected.length;i++) {
					VideoStreamDescriptor streamDescriptor=(VideoStreamDescriptor)selected[i];
					enabled |= streamDescriptor.getCodec().startsWith(StreamIdentifier.VC1VideoAnalyzer.CODEC_NAME) && streamDescriptor.getInterlaced()==1;
				}
				hdDvdVc1ToBluRayCheckBox.setEnabled(enabled);
			}
		};
		audioListBox.addListSelectionListener(videoListSelectionListener);
		getContentPane().repaint();
		
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				Rectangle bounds=getBounds();
				prefs.putInt(PREFS_X, bounds.x);
				prefs.putInt(PREFS_Y, bounds.y);
				prefs.putInt(PREFS_WIDTH, bounds.width);
				prefs.putInt(PREFS_HEIGHT, bounds.height);
				dispose();
			}
		});
	}
	
	private JobStreamI getJobStream(Job job, File[] files) {
		if (files[0].getName().toLowerCase().endsWith(".wtv")) {
			return new WtvFileStream(job, files[0]);
		} else {
			return new FileStream(job, files);
		}
	}
}
