package util;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import common.Util;
import common.matroska.MatroskaConstants;
import common.matroska.MatroskaDecoder;
import common.matroska.MatroskaDecoder.IdInfo;
import common.matroska.MatroskaDecoder.MClass;

public class MatroskaAnalyzer implements MatroskaConstants {
	
	private FileChannel channel;
	//private MatroskaDecoder decoder=new MatroskaDecoder(null);
	private ByteBuffer intBuffer=ByteBuffer.allocateDirect(8);
	private ByteBuffer blockBuffer=ByteBuffer.allocateDirect(16);
	private ByteBuffer binaryBuffer=ByteBuffer.allocateDirect(16);
	
	private MatroskaDecoder intDecoder=new MatroskaDecoder(intBuffer);
	private byte stringBytes[]=new byte[128];
	private ByteBuffer stringBuffer=ByteBuffer.wrap(stringBytes);
	private long timecode;
	
	
	
	public static void main(String args[]) {
		String defaultName="C:/RemuxTool/matrix.mkv";
		//="H:/video/pin.mkv";
		String fileName=args.length==0?defaultName:args[0];
		new MatroskaAnalyzer().analyze(fileName);
	}
	
	public void analyze(String fileName) {
		try {
			channel=new RandomAccessFile(fileName,"r").getChannel();
			parseLevel(0, channel.size());
		} catch (IOException e) {
			
		}
	}
	
	private void parseLevel(int level, long size) throws IOException {
		long end=channel.position()+size;
		StringBuffer prefix=new StringBuffer(level+1);
		for (int i=0;i<level;i++) {
			prefix.append(' ');
		}
		prefix.append('+');
		
		while (channel.position() < end) {
			StringBuffer sb=new StringBuffer(prefix.length() + 20);
			sb.append(prefix);
			
			IdInfo info=MatroskaDecoder.getNextChunkInfo(channel);
			MClass mClass=MatroskaDecoder.getMClass(info.id);
			if (mClass==null) {
				sb.append("Unknown: 0x");
				sb.append(Integer.toString(info.id, 16));
				sb.append(" Len: 0x");
				sb.append(Long.toString(info.len, 16));
				System.out.println(sb);
				channel.position(channel.position() + info.len + info.idLen);
			} else {
				channel.position(channel.position() + info.idLen);
				sb.append(mClass.name);
				/*sb.append("(0x");
				sb.append(Long.toString(channel.position()-info.idLen, 16));
				sb.append(')');*/
				sb.append(": ");
				
				if (mClass.type==MClass.TYPE_MASTER) {
					sb.append("0x");
					sb.append(Long.toString(info.len, 16));
					System.out.println(sb);
					parseLevel(level+1, info.len);
					
				} else {
					switch (mClass.type) {
					case MClass.TYPE_UINT:
						sb.append("0x");
						sb.append(Long.toString(getULong((int)info.len), 16));
						break;
					case MClass.TYPE_SIMPLE_BLOCK:
						blockBuffer.position(0);
						blockBuffer.limit((int)Math.min(blockBuffer.capacity(), info.len));
						channel.read(blockBuffer);
						blockBuffer.rewind();
						int track=blockBuffer.get() & 0x7f;
						int offset=blockBuffer.getShort();
						int flags=blockBuffer.get() & 0xff;
						int keyFrame=(flags >>> 7);
						int lace=(flags >>> 1) & 0x3;
						sb.append("Track: 0x");
						sb.append(Integer.toString(track, 16));
						if (keyFrame==1) {
							sb.append('*');
						}
						if (offset != 0) {
							sb.append(' ');
							sb.append(Util.timeToString(timecode + offset));
						}
						if (lace > 0) {
							int frames=(blockBuffer.get() & 0xff) + 1;
							sb.append(" Frames: ");
							sb.append(frames);
							/*if ((lace & 1)==1) {
								for (int i=0;i<frames;i++) {
									
								}
							}*/
						}
						channel.position(channel.position() + info.len - blockBuffer.limit());
						break;
					case MClass.TYPE_TIMECODE:
						timecode=getULong((int)info.len);
						sb.append(Util.timeToString(timecode));
						break;
					case MClass.TYPE_STRING:
					case MClass.TYPE_UTF8:
						stringBuffer.limit((int)info.len);
						stringBuffer.position(0);
						channel.read(stringBuffer);
						sb.append(new String(stringBytes, 0, stringBuffer.limit(),"UTF-8"));
						break;
					case MClass.TYPE_BINARY:
						binaryBuffer.position(0);
						binaryBuffer.limit((int)Math.min(binaryBuffer.capacity(), info.len));
						channel.read(binaryBuffer);
						binaryBuffer.rewind();
						sb.append("Binary 0x");
						sb.append(Long.toString(info.len, 16));
						sb.append('-');
						while (binaryBuffer.hasRemaining()) {
							Util.appendHex(binaryBuffer.get(), sb);
							sb.append(' ');
						}
						channel.position(channel.position() + info.len - binaryBuffer.limit());
						break;
					case MClass.TYPE_DATE:
						intBuffer.limit(8);
						intBuffer.position(0);
						channel.read(intBuffer);
						intBuffer.rewind();
						long time=intBuffer.getLong()/1000000 + MatroskaDecoder.MATROSKA_EPOCH.getTimeInMillis();
						sb.append(new Date(time));
						break;
					case MClass.TYPE_FLOAT:
						intBuffer.limit((int)info.len);
						intBuffer.position(0);
						channel.read(intBuffer);
						intBuffer.rewind();
						if (info.len==4) {
							sb.append(intBuffer.getFloat());
						} else {
							sb.append(intBuffer.getDouble());
						}
						break;
					case MClass.TYPE_SINT:
						intBuffer.limit((int)info.len);
						intBuffer.position(0);
						channel.read(intBuffer);
						intBuffer.rewind();
						sb.append(intDecoder.getSLong((int)info.len));
						break;
					}
					System.out.println(sb);
				}
			}
		}
	}
	private long getULong(int len) throws IOException {
		intBuffer.limit(len);
		intBuffer.position(0);
		channel.read(intBuffer);
		intBuffer.rewind();
		return intDecoder.getULong(len);
		
	}
}
